import { XMLElem, type IToXML } from "@/util/XmlElem";
import type { NamespaceTable } from "./NameSpaceTable";
import { assert } from "@/util/assert";
import { NodeId, coerceNodeId } from "./NodeId";

export class UAReference implements IToXML{
    constructor(public referenceType: string,
                public ref: NodeId,
                public isForward: boolean) {
    }


    reIndex(nst: NamespaceTable, onst: NamespaceTable) {
        const nsName=onst.getUri(this.ref.namespace);
        assert(nsName!=undefined)
        const newIndex=nst.getIndex(nsName);
        assert(newIndex!=undefined)
        this.ref.namespace=newIndex;    
    }

    toXML(): XMLElem {
        return new XMLElem('Reference', this.ref.toString())
            .attr('ReferenceType', this.referenceType.toString())
            .attr('IsForward',this.isForward);
        
    }

    static fromXML(uaReference: any): UAReference {
        return new UAReference(uaReference['@_ReferenceType'], 
                                coerceNodeId(uaReference['#text']), 
                                uaReference['@_IsForward']!="false");
    }
}
